/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package uk.ac.bbsrc.babraham.ChipMonk;


import java.awt.BorderLayout;
import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.UIManager;
import javax.swing.filechooser.FileFilter;

import uk.ac.bbsrc.babraham.ChipMonk.DataParsers.ChipMonkParser;
import uk.ac.bbsrc.babraham.ChipMonk.DataParsers.DataParserListener;
import uk.ac.bbsrc.babraham.ChipMonk.DataParsers.DataParserProgressDialog;
import uk.ac.bbsrc.babraham.ChipMonk.DataParsers.GenePixParser;
import uk.ac.bbsrc.babraham.ChipMonk.DataParsers.GenericParser;
import uk.ac.bbsrc.babraham.ChipMonk.DataParsers.NimblegenParser;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Chromosome;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataChangeListener;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataCollection;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataGroup;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataSet;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataStore;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Feature;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Genome;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.ProbeList;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.ProbeSetChangeListener;
import uk.ac.bbsrc.babraham.ChipMonk.DataWriters.ChipMonkDataWriter;
import uk.ac.bbsrc.babraham.ChipMonk.DataWriters.DataWriterListener;
import uk.ac.bbsrc.babraham.ChipMonk.DataWriters.DataWriterProgressDialog;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.GenomeSelector;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.ProbeListViewer;
import uk.ac.bbsrc.babraham.ChipMonk.Displays.StatusPanel;
import uk.ac.bbsrc.babraham.ChipMonk.Displays.ChromosomeViewer.ChromosomeViewer;
import uk.ac.bbsrc.babraham.ChipMonk.Displays.DataViewer.DataViewer;
import uk.ac.bbsrc.babraham.ChipMonk.Displays.FeatureViewer.FeatureViewer;
import uk.ac.bbsrc.babraham.ChipMonk.Displays.GenomeViewer.GenomeViewer;
import uk.ac.bbsrc.babraham.ChipMonk.Displays.MAPlot.MAPlot;
import uk.ac.bbsrc.babraham.ChipMonk.Displays.ScatterPlot.ScatterPlot;
import uk.ac.bbsrc.babraham.ChipMonk.GenomeParsers.EMBLParser;
import uk.ac.bbsrc.babraham.ChipMonk.GenomeParsers.GenomeParserListener;
import uk.ac.bbsrc.babraham.ChipMonk.GenomeParsers.GenomeParserProgressDialog;
import uk.ac.bbsrc.babraham.ChipMonk.Network.GenomeDownloader;
import uk.ac.bbsrc.babraham.ChipMonk.Network.GenomeDownloaderListener;
import uk.ac.bbsrc.babraham.ChipMonk.Network.GenomeDownloaderProgressDialog;
import uk.ac.bbsrc.babraham.ChipMonk.Normalisation.GlobalNormalisation;
import uk.ac.bbsrc.babraham.ChipMonk.Normalisation.Normalisation;
import uk.ac.bbsrc.babraham.ChipMonk.Normalisation.NormalisationProgressDialog;
import uk.ac.bbsrc.babraham.ChipMonk.Normalisation.NormalisationProgressListener;
import uk.ac.bbsrc.babraham.ChipMonk.Normalisation.NormalisationRunner;
import uk.ac.bbsrc.babraham.ChipMonk.Normalisation.ResetNormalisation;
import uk.ac.bbsrc.babraham.ChipMonk.Preferences.ChipMonkPreferences;

public class ChipMonkApplication extends JFrame implements GenomeParserListener, DataParserListener, NormalisationProgressListener, GenomeDownloaderListener, DataWriterListener, DataChangeListener, ProbeSetChangeListener{

	public static final String VERSION = "1.2.3";
	
	private Genome genome;
	private ChipMonkMenu menu;
	private GenomeViewer genomeViewer = null;
	private DataViewer dataViewer = null;
	private ChromosomeViewer chromosomeViewer;
	private JSplitPane mainPane;
	private JSplitPane topPane;
	private StatusPanel statusPanel;
	private DataCollection dataCollection = new DataCollection();
	private Vector drawnFeatureTypes = new Vector();
	private Vector drawnDataStores = new Vector();
	private Normalisation [] normalisationModules = new Normalisation[] {new ResetNormalisation(),new GlobalNormalisation()};
	private File currentFile = null;
	
	// Flag to check if anything substantial has changed since the file
	// was last loaded/saved.
	private boolean changesWereMade = false;
	// Flag used only when saving before shutting down (save on exit)
	private boolean shuttingDown = false;
	
	public static void main(String[] args) {

		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {}
		
		try {
			ChipMonkApplication a = new ChipMonkApplication();
			if (args.length > 0) {
				File f = new File(args[0]);
				a.loadProject(f);
			}
		}
		catch (Exception e) {
			JOptionPane.showMessageDialog(null,"Unexptected Error:"+e.getMessage(),"Error",JOptionPane.ERROR_MESSAGE);
		}
	}
	
	

	public ChipMonkApplication (){		
		setTitle("ChIP Monk");
		setSize(800,600);
		menu = new ChipMonkMenu(this);
		setJMenuBar(menu);
		setLocationRelativeTo(null);
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		setIconImage(new ImageIcon(ClassLoader.getSystemResource("uk/ac/bbsrc/babraham/ChipMonk/Resources/monk_logo.png")).getImage());
		
		mainPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		
		topPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		
		mainPane.setTopComponent(topPane);

		getContentPane().setLayout(new BorderLayout());
		
		getContentPane().add(mainPane,BorderLayout.CENTER);
		
		statusPanel = new StatusPanel();
		
		getContentPane().add(statusPanel,BorderLayout.SOUTH);
		
		setVisible(true);
		mainPane.setDividerLocation((double)0.25);
		topPane.setDividerLocation((double)0.25);
	}
		
	private void changesWereMade () {
//		System.err.println("Changes were made...");
//		new Throwable().printStackTrace();
		changesWereMade = true;
	}
	public void resetChangesWereMade () {
//		System.err.println("Changes were reset...");
//		new Throwable().printStackTrace();
		changesWereMade = false;
	}
	
	
	public void dispose () {
		// We're overriding this so we can catch the application being
		// closed by the X in the corner.  We need to offer the opportunity
		// to save if they've changed anything.

		// We'll already have been made invisible by this stage, so make
		// us visible again in case we're hanging around.
		setVisible(true);
		
		// Check to see if the user has made any changes they might
		// want to save
		if (changesWereMade) {
			int answer = JOptionPane.showOptionDialog(this,"You have made changes which were not saved.  Do you want to save before exiting?","Save before exit?",0,JOptionPane.QUESTION_MESSAGE,null,new String [] {"Save and Exit","Exit without Saving","Cancel"},"Save");

			switch (answer){
			case 0: 
				shuttingDown = true;
				saveProject();
				return;
			case 1:
				break;
			case 2:
				return;
			}
		}

		setVisible(false);
		super.dispose();
		System.exit(0);
		
	}
	public void startNewProject () {
		new GenomeSelector(this);
	}
	
	public void wipeAllData () {
		currentFile = null;
		topPane.setRightComponent(null);
		topPane.setLeftComponent(null);
		mainPane.setBottomComponent(null);
		genome = null;
		genomeViewer = null;
		dataViewer = null;
		chromosomeViewer = null;
		dataCollection = new DataCollection();
		dataCollection.addDataChangeListener(this);
		dataCollection.probeSet().addProbeSetChangeListener(this);
		drawnFeatureTypes = new Vector();
		drawnDataStores = new Vector();
		normalisationModules = new Normalisation[] {new ResetNormalisation()};
	}
	
	
	public void importNimblegen () {
		JFileChooser chooser = new JFileChooser(ChipMonkPreferences.getInstance().getDataLocation());
		chooser.setMultiSelectionEnabled(true);
		chooser.setFileFilter(new FileFilter() {
		
			public String getDescription() {
				return "All Pair Files";
			}
		
			public boolean accept(File f) {
				if (f.isDirectory() || f.getName().toLowerCase().endsWith("pair.txt")) {
					return true;
				}
				else {
					return false;
				}
			}
		
		});
		
		int result = chooser.showOpenDialog(this);
		if (result == JFileChooser.CANCEL_OPTION) return;

		File [] files = chooser.getSelectedFiles();
		
		NimblegenParser parser = new NimblegenParser(this);
		
		parser.addDataParserListener(this);
		parser.addDataParserListener(new DataParserProgressDialog(this));
		
		for (int i=0;i<files.length;i++) {
//			System.out.println("Launching parse of "+files[i].getName());
			parser.parseFile(files[i]);
		}

		
	}
	
	public void importProbeList () {
		JFileChooser chooser = new JFileChooser(ChipMonkPreferences.getInstance().getSaveLocation());
		chooser.setMultiSelectionEnabled(false);
		chooser.setFileFilter(new FileFilter() {
		
			public String getDescription() {
				return "Text Files";
			}
		
			public boolean accept(File f) {
				if (f.isDirectory() || f.getName().toLowerCase().endsWith(".txt")) {
					return true;
				}
				else {
					return false;
				}
			}
		
		});
		
		int result = chooser.showOpenDialog(this);
		if (result == JFileChooser.CANCEL_OPTION) return;

		File file = chooser.getSelectedFile();
		
		try {
			BufferedReader br = new BufferedReader(new FileReader(file));
			
			String line;
			String [] sections;
			
			ProbeList newList = new ProbeList(dataCollection.probeSet(),file.getName(),"No value");
			
			boolean firstLine = true;
			while ((line = br.readLine())!=null) {
				if (line.length() == 0) continue;
				if (line.charAt(0) == '#')continue;  // Ignore comments.
				
				sections = line.split("\\t");
				if (dataCollection.probeSet().containsProbeWithName(sections[0])) {
					newList.addProbe(dataCollection.probeSet().getProbeWithName(sections[0]), null);
				}
				else {
					if (!firstLine) {
						throw new ChipMonkException("No probe found with name '"+sections[0]+"'");
					}
				}
				firstLine = false;
			}
			
			dataCollection.probeSet().addProbeList(newList);
		} 
		catch (FileNotFoundException e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(this, "Couldn't read file", "Error", JOptionPane.ERROR_MESSAGE);
		}
		catch (IOException e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(this, "Error reading file", "Error", JOptionPane.ERROR_MESSAGE);
		}
		catch (ChipMonkException e) {
			JOptionPane.showMessageDialog(this, e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
		}
	}
	
	public void importGeneric () {
		new GenericParser(this);
	}
	
	public void importGenePix () {
		JFileChooser chooser = new JFileChooser(ChipMonkPreferences.getInstance().getDataLocation());
		chooser.setMultiSelectionEnabled(false);
		chooser.setFileFilter(new FileFilter() {
		
			public String getDescription() {
				return "Genome Postion Files";
			}
		
			public boolean accept(File f) {
				if (f.isDirectory() || f.getName().toLowerCase().endsWith(".txt")) {
					return true;
				}
				else {
					return false;
				}
			}
		
		});
		
		int result = chooser.showOpenDialog(this);
		if (result == JFileChooser.CANCEL_OPTION) return;

		File genomeFile = chooser.getSelectedFile();

		chooser.setMultiSelectionEnabled(true);
		chooser.setFileFilter(new FileFilter() {
		
			public String getDescription() {
				return "GenePix Files";
			}
		
			public boolean accept(File f) {
				if (f.isDirectory() || f.getName().toLowerCase().endsWith(".gpr")) {
					return true;
				}
				else {
					return false;
				}
			}
		
		});
		
		result = chooser.showOpenDialog(this);
		if (result == JFileChooser.CANCEL_OPTION) return;

		File [] dataFiles = chooser.getSelectedFiles();

			
		GenePixParser parser = new GenePixParser(this);
		
		parser.addDataParserListener(this);
		parser.addDataParserListener(new DataParserProgressDialog(this));
		
		parser.parseFile(genomeFile,dataFiles);
		
	}
	
	public void saveProjectAs() {
		JFileChooser chooser = new JFileChooser(ChipMonkPreferences.getInstance().getSaveLocation());
		chooser.setMultiSelectionEnabled(false);
		chooser.setFileFilter(new FileFilter() {
		
			public String getDescription() {
				return "ChIPMonk files";
			}
		
			public boolean accept(File f) {
				if (f.isDirectory() || f.getName().toLowerCase().endsWith(".cmk")) {
					return true;
				}
				else {
					return false;
				}
			}
		
		});
		
		int result = chooser.showSaveDialog(this);
		if (result == JFileChooser.CANCEL_OPTION) return;

		File file = chooser.getSelectedFile();
		if (! file.getPath().toLowerCase().endsWith(".cmk")) {
			file = new File(file.getPath()+".cmk");
		}

		// Check if we're stepping on anyone's toes...
		if (file.exists()) {
			int answer = JOptionPane.showOptionDialog(this,file.getName()+" exists.  Do you want to overwrite the existing file?","Overwrite file?",0,JOptionPane.QUESTION_MESSAGE,null,new String [] {"Overwrite and Save","Cancel"},"Overwrite and Save");

			if (answer > 0) {
				saveProjectAs(); // Let them try again
				return;
			}
		}
		
		currentFile = file;
		saveProject(file);
	}
	
	public void exportCurrentView() {
		JFileChooser chooser = new JFileChooser(ChipMonkPreferences.getInstance().getSaveLocation());
		chooser.setMultiSelectionEnabled(false);
		chooser.setFileFilter(new FileFilter() {
		
			public String getDescription() {
				return "PNG Files";
			}
		
			public boolean accept(File f) {
				if (f.isDirectory() || f.getName().toLowerCase().endsWith(".png")) {
					return true;
				}
				else {
					return false;
				}
			}
		
		});
		
		int result = chooser.showSaveDialog(this);
		if (result == JFileChooser.CANCEL_OPTION) return;

		File file = chooser.getSelectedFile();
		if (! file.getPath().toLowerCase().endsWith(".png")) {
			file = new File(file.getPath()+".png");
		}

		
		try {					
			ImageIO.write((BufferedImage)(chromosomeViewer.getCurrentView()),"PNG",file);
		}

		catch (IOException e) {
			JOptionPane.showMessageDialog(this,"Error saving image: "+e.getMessage(),"Error",JOptionPane.ERROR_MESSAGE);
		}
	}
	
	public void saveProject () {
		if (currentFile == null) {
			saveProjectAs();
		}
		else {
			saveProject(currentFile);
		}
	}
	
	public void saveProject (File file) {
		
		ChipMonkDataWriter writer = new ChipMonkDataWriter();
		
		writer.addDataWriterListener(new DataWriterProgressDialog(this));
		writer.addDataWriterListener(this);
		
		writer.writeData(this,file);
		
		setTitle("ChIP Monk ["+file.getName()+"]");
	}
	
	public void loadProject () {
		JFileChooser chooser = new JFileChooser(ChipMonkPreferences.getInstance().getSaveLocation());
		chooser.setMultiSelectionEnabled(false);
		chooser.setFileFilter(new FileFilter() {
		
			public String getDescription() {
				return "ChIPMonk files";
			}
		
			public boolean accept(File f) {
				if (f.isDirectory() || f.getName().toLowerCase().endsWith(".cmk")) {
					return true;
				}
				else {
					return false;
				}
			}
		
		});
		
		int result = chooser.showOpenDialog(this);
		if (result == JFileChooser.CANCEL_OPTION) return;

		File file = chooser.getSelectedFile();
	
		loadProject(file);
	}

	public void loadProject (File file) {

		if (file == null) return;
		
		wipeAllData();
		
		currentFile = file;
		
		ChipMonkParser parser = new ChipMonkParser(this);
		
		parser.addDataParserListener(this);
		
		DataParserProgressDialog dppd = new DataParserProgressDialog(this);
		parser.addDataParserListener(dppd);
		parser.parseFile(file);
		dppd.requestFocus();
		setTitle("ChIP Monk ["+file.getName()+"]");

	}

	
	public void downloadGenome (String species, String assembly, int size) {
		GenomeDownloader d = new GenomeDownloader();
		d.addGenomeDownloaderListener(this);
		GenomeDownloaderProgressDialog pd = new GenomeDownloaderProgressDialog(this);
		d.addGenomeDownloaderListener(pd);
		d.downloadGenome(species,assembly,size);
		pd.requestFocus();
	}
	
	public void loadGenome (File baseLocation) {
		// Testing
		EMBLParser parser = new EMBLParser();
		GenomeParserProgressDialog pd = new GenomeParserProgressDialog(this);
		parser.addGenomeParserListener(pd);
		parser.addGenomeParserListener(this);
		parser.parseGenome(baseLocation);
		pd.requestFocus();
	}
	

	// Methods needed so that we can listen to the load progress of
	// a new genome
	public void genomeUpdateProgress(int currentPos, int totalPos, String message) {}
	public void genomeExceptionReceived(Exception e) {}

	public void genomeProcessingFinished(Genome g) {
		genome = g;
		
		// By default we show all available feature types in
		// the chromosome view.
		String [] t = g.listAvailableFeatureTypes();
		for (int i=0;i<t.length;i++) {
			drawnFeatureTypes.add(t[i]);
		}
		genomeViewer = new GenomeViewer(genome,this);
		topPane.setRightComponent(genomeViewer);
		dataViewer = new DataViewer(this);
		topPane.setLeftComponent(new JScrollPane(dataViewer));
		validate();
		
		displayChromosome((genome.getAllChromosomes())[0],1);
		menu.genomeLoaded();
		
	}
	// End of the GenomeProgressListener methods.
	
	public ChromosomeViewer chromosomeViewer () {
		return chromosomeViewer;
	}
	
	public DataViewer dataViewer () {
		return dataViewer;
	}
			
	public boolean dataStoreIsDrawn (DataStore d) {
		if (drawnDataStores.contains(d)) {
			return true;
		}
		return false;
	}
	public void addToDrawnDataStores (DataStore d) {
		// Remember that we changed something
		changesWereMade();
		if (! drawnDataStores.contains(d)) {
			drawnDataStores.add(d);
			chromosomeViewer.tracksUpdated();
		}
	}
	
	public void removeFromDrawnDataStores (DataStore d) {
		// Remember that we changed something
		changesWereMade();
		if (drawnDataStores.contains(d)) {
			drawnDataStores.remove(d);
			chromosomeViewer.tracksUpdated();
		}
	}
	
	public void setDrawnDataStores (DataStore [] d) {
		// Remember that we changed something
		changesWereMade();
		
		drawnDataStores.removeAllElements();
		for (int i=0;i<d.length;i++) {
			drawnDataStores.add(d[i]);
		}
		chromosomeViewer.tracksUpdated();
	}
	
	public void displayChromosome (Chromosome c, int midPoint) {
		chromosomeViewer = new ChromosomeViewer(this,c);
		mainPane.setBottomComponent(chromosomeViewer);
		
		// We need to validate here so that the chromosomeViewer
		// figures out what size it's supposed to be so that we
		// can move its scroll bar to the correct point.
		validate();
		chromosomeViewer.setMidPoint(midPoint);
	}
	
	public void setStatusText (String text) {
		statusPanel.setText(text);
	}
	
	public void setChromosomeView (Chromosome c, int start, int end) {
		if (genomeViewer != null) {
			genomeViewer.setView(c,start,end);
		}
	}
	
	public String [] drawnFeatureTypes() {
		return (String [])drawnFeatureTypes.toArray(new String[0]);
	}
	
	public void setDrawnFeatureTypes (String [] features) {
		drawnFeatureTypes.removeAllElements();
		for (int i=0;i<features.length;i++) {
			drawnFeatureTypes.add(features[i]);
		}
		chromosomeViewer.tracksUpdated();
	}
	
	public DataStore [] drawnDataStores () {
		return (DataStore [])drawnDataStores.toArray(new DataStore[0]);
	}
	
	public void showFeatureDetails(Feature f) {
		new FeatureViewer(f,this);
	}
	
	public void showProbeListDetails (ProbeList p) {
		new ProbeListViewer(p,this);
	}
	
	public Genome genome () {
		return genome;
	}
		
	public DataCollection dataCollection () {
		return dataCollection;
	}

	// Methods for DataProgressListener interface
	public void dataUpdateProgress(int c, int t, String message) {}

	public void dataExceptionReceived(Exception e) {
		e.printStackTrace();
	}
	public void dataWarningReceived(Exception e) {
		// We can ignore these, they're caught in the progress dialog
	}

	public void dataProcessingFinished(boolean requiresNormalisation) {
		dataViewer = new DataViewer(this);
		topPane.setLeftComponent(new JScrollPane(dataViewer));
		if (dataCollection.getAllDataSets().length>0)
			menu.dataLoaded();
		
		if (requiresNormalisation) 
			applyNormalisation();
	}
	// End of DataProgressListener
	
	public void showMAPlot (DataStore d) {
		new MAPlot(this,dataCollection,d);
	}
	
	public void showScatterPlot (DataStore d) {
		new ScatterPlot(this,dataCollection,d);
	}
	
	public void applyNormalisation () {
		NormalisationRunner r = new NormalisationRunner(normalisationModules);
		r.addNormalisationProgressListener(this);
		r.addNormalisationProgressListener(new NormalisationProgressDialog(this));
		r.normalise(dataCollection);
	}
		
	// This method should only be used during the restoring
	// of a saved project.  It's intended for use where you 
	// need to change the normalisation modules, but you don't
	// yet have all of the data loaded.
	public void setNormalisationModulesWithoutApplying (Normalisation [] m) {
		normalisationModules = m;
	}
	
	public void setNormalisationModules (Normalisation [] m) {
		normalisationModules = m;
		applyNormalisation();
	}
	
	public Normalisation [] getNormalisationModules () {
		return normalisationModules;
	}


	// These methods are from Normalisation progress listener
	public void updateNormalisationProgress(String message) {
	}

	public void normalisationComplete() {
		// Need to refresh any data tracks which are being displayed
		chromosomeViewer.tracksUpdated();
		
		// This counts as a modification, so set the flag
		changesWereMade();
	}
	
	public void normalisationExceptionReceived(Exception e) {
	}
	// End of normalisation progress listener methods.



	// GenomeDownloaderListener methods.
	public void downloadUpdateProgress(int currentPos, int totalPos, String message) {
	}
	public void downloadExceptionReceived(Exception e) {
	}

	public void downloadProcessingFinished() {
		startNewProject();
	}
	// End of genomeDownloaderListener methods



	// These methods are from DataWriterListener.  We only use them
	// when we're shutting down and need to write the data before
	// we go.
	public void dataWriterExceptionReceived(Exception e) {
	}
	public void dataWriterProgressUpdate(String message, int current, int total) {
	}
	public void dataWritingComplete() {
		// Since we've just saved we can reset the changes flag
		resetChangesWereMade();
		
		// We might have been called by a previous shutdown
		// operation, in which case we need to send them
		// back to shut down.
		if (shuttingDown) {
			shuttingDown = false;
			dispose();
		}
	}



	public void dataGroupAdded(DataGroup g) {
		changesWereMade();
	}

	public void dataGroupRemoved(DataGroup g) {
		removeFromDrawnDataStores(g);
		changesWereMade();
	}

	public void dataSetAdded(DataSet d) {
		changesWereMade();
	}	

	public void dataSetRemoved(DataSet d) {
		removeFromDrawnDataStores(d);
		changesWereMade();
	}

	public void probeListAdded(ProbeList l) {
		changesWereMade();
	}

	public void probeListRemoved(ProbeList l) {
		changesWereMade();
	}

	public void dataGroupRenamed(DataGroup g) {
		changesWereMade();
	}

	public void dataGroupSamplesChanged(DataGroup g) {
		changesWereMade();
	}

	public void dataSetRenamed(DataSet d) {
		changesWereMade();
	}

	public void probeListRenamed(ProbeList l) {
		changesWereMade();
	}

}
