/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Displays.ChromosomeViewer;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JPanel;

import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Feature;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Location;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.SubLocation;

public class ChromosomeFeatureTrack extends JPanel {
	
	private ChromosomeViewer viewer;
	private Feature activeFeature = null;
	private Feature [] features;
	private String featureName;
	private int x;
	private int width;
	private int fullWidth;
	private int height;
	private Vector drawnFeatures = new Vector();
	
	public ChromosomeFeatureTrack (ChromosomeViewer viewer, String featureName) {
		this.viewer = viewer;
		this.featureName = featureName;
		features = viewer.chromosome().getFeaturesForType(featureName);
		addMouseMotionListener(new FeatureListener());
		addMouseListener(new FeatureListener());
	}
		
	public void paintComponent (Graphics g) {

		super.paintComponent(g);
		((Graphics2D)g).setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		drawnFeatures = new Vector();
		fullWidth = getWidth();
		height = getHeight();
		Rectangle r = getVisibleRect();
		x = r.x;
		width = r.width;
		
		
		if (viewer.getIndex(this)%2 == 0) {
			g.setColor(new Color(180,180,255));
		}
		else {
			g.setColor(new Color(220,220,255));
		}
		
		g.fillRect(x,0,width,height);
		
		if (viewer.makingSelection()) {
			int selStart = viewer.selectionStart();
			int selEnd = viewer.selectionEnd();
			int useStart = (selEnd > selStart) ? selStart : selEnd;
			int selWidth = selEnd-selStart;
			if (selWidth < 0) selWidth = 0-selWidth;
			g.setColor(Color.GREEN);
			g.fillRect(useStart,0,selWidth,height);
		}
				
		// Now go through all the features figuring out whether they
		// need to be displayed
		int startBp = pixelToBp(x);
		int endBp = pixelToBp(x+width);
		for (int i=0;i<features.length;i++) {
//			System.out.println("Looking at feature "+featureName+" Start: "+features[i].location().start()+" End: "+features[i].location().end()+" Start:"+startBp+" End:"+endBp);
			if (features[i].location().end() > startBp && features[i].location().start() < endBp) {
				// We always draw the active feature last so skip it here.
				if (features[i] != activeFeature)
					drawFeature(features[i],g);
			}
		}
		
		// Finally redraw the active feature so it always goes on top
		if (activeFeature != null)
			drawFeature(activeFeature,g);
		
		// Lastly draw the name of the track
		g.setColor(Color.GRAY);
		g.drawString(featureName,x+2,height-2);

	}
	
	private void drawFeature (Feature f, Graphics g) {
		
		if (f.location().strand() == Location.FORWARD) {
			g.setColor(Color.RED);
		}
		else {
			g.setColor(Color.BLUE);
		}
		
		if (f == activeFeature) {
			g.setColor(Color.YELLOW);
		}
		
		// If there's space we'll put a label on the track as
		// well as the feature.
		boolean drawLabel = false;
		int yBoxStart = 2;
		int yBoxEnd = height-2;
		int yText = 0;
		if (height > 25) {
			drawLabel = true;
			yBoxStart = 2;
			yBoxEnd = height-14;
			yText = height-2;
		}
		
		int wholeXStart = bpToPixel(f.location().start());
		int wholeXEnd = bpToPixel(f.location().end());
		
		if (wholeXEnd - wholeXStart < 2) {
			wholeXStart = wholeXEnd - 2;
		}
		
//		drawnFeatures.add(new DrawnFeature(wholeXStart, wholeXEnd, f));
		
		if (wholeXEnd-wholeXStart < 10) {
			// Don't bother drawing sublocations
			g.fillRect(wholeXStart,yBoxStart,(wholeXEnd-wholeXStart),yBoxEnd-yBoxStart);
			drawnFeatures.add(new DrawnFeature(wholeXStart, wholeXEnd, f));
		}

		else {
			// Draw out the individual sublocations.
			SubLocation [] l = f.location().subLocations();
			int lastX = -1;
			for (int i=0;i<l.length;i++) {
				int xStart = bpToPixel(l[i].start());
				int xEnd = bpToPixel(l[i].end());
				
				// Make sure something shows up.
				if ((xEnd-xStart)<2) {
					xStart = xEnd -2;
				}
				
				drawnFeatures.add(new DrawnFeature(xStart, xEnd, f));
				
				// Draw an intron line if this isn't the first subLocation
				if (i>0) {
//					g.drawLine(lastX,yBoxStart+(yBoxEnd-yBoxStart)/2,xStart,yBoxStart+(yBoxEnd-yBoxStart)/2);
					if (f.location().strand() == 1) {
						// We're going forward
						g.drawLine(lastX,yBoxStart+(yBoxEnd-yBoxStart)/3,xStart,yBoxStart+(yBoxEnd-yBoxStart)/2);
						g.drawLine(lastX, yBoxEnd-(yBoxEnd-yBoxStart)/3, xStart, yBoxStart+(yBoxEnd-yBoxStart)/2);
					}
					else {
						// We're going backwards
						g.drawLine(xStart,yBoxStart+(yBoxEnd-yBoxStart)/3,lastX,yBoxStart+(yBoxEnd-yBoxStart)/2);
						g.drawLine(xStart, yBoxEnd-(yBoxEnd-yBoxStart)/3, lastX, yBoxStart+(yBoxEnd-yBoxStart)/2);
					}
				}
				
				// Draw a box around the feature
				g.fillRect(xStart,yBoxStart,(xEnd-xStart),yBoxEnd-yBoxStart);
				
				lastX = xEnd;
			}	
		}
		if (drawLabel && (f==activeFeature || viewer.showAllLables())) {
			g.setColor(Color.DARK_GRAY);
			g.drawString(f.type()+":"+f.name(),wholeXStart,yText);
		}
	}
	private int pixelToBp (int x) {
		return (int)(((double)x/fullWidth)*viewer.chromosome().length());
	}
	
	private int bpToPixel (int bp) {
		return (int)(((double)bp/viewer.chromosome().length())*fullWidth);
		
	}
	
	private class DrawnFeature {
		
		public int start;
		public int end;
		public Feature feature;
		
		public DrawnFeature (int start, int end, Feature feature) {
			this.start = start;
			this.end = end;
			this.feature = feature;
		}
		
		public boolean isInFeature (int x) {
			if (x >=start && x <= end) {
				return true;
			}
			else {
				return false;
			}
		}
	}
	
	private class FeatureListener implements MouseMotionListener, MouseListener {

		public void mouseDragged(MouseEvent me) {
			viewer.setSelectionEnd(me.getX());
		}

		public void mouseMoved(MouseEvent me) {
			int x = me.getX();
			Enumeration e = drawnFeatures.elements();
			while (e.hasMoreElements()) {
				DrawnFeature f = (DrawnFeature)e.nextElement();
				if (f.isInFeature(x)) {
					if (activeFeature != f.feature) {
						viewer.application().setStatusText(" "+f.feature.type()+": "+f.feature.name());
						activeFeature = f.feature;
						repaint();
					}
				}
			}
		}

		public void mouseClicked(MouseEvent me) {
			if ((me.getModifiers() & InputEvent.BUTTON3_MASK) == InputEvent.BUTTON3_MASK) {
				viewer.zoomOut();
				return;
			}
			if (me.getClickCount() >= 2) {
				if (activeFeature != null) {
					viewer.application().showFeatureDetails(activeFeature);
				}
			}
		}
		
		public void mousePressed(MouseEvent me) {
			viewer.setMakingSelection(true);
			viewer.setSelectionStart(me.getX());
			viewer.setSelectionEnd(me.getX());
		}

		public void mouseReleased(MouseEvent me) {
			viewer.setMakingSelection(false);

			int width = viewer.selectionEnd() - viewer.selectionStart();
			if (width < 0) {
				width = 0-width;
			}
			
			// Stop people from accidentally making really short selections
			if (width < 5) return;
			
						
			viewer.setView(pixelToBp(viewer.selectionStart()),pixelToBp(viewer.selectionEnd()));
		}

		public void mouseEntered(MouseEvent arg0) {
			if (features.length>0)
				viewer.application().setStatusText(" "+features[0].type());
		}

		public void mouseExited(MouseEvent arg0) {
			activeFeature = null;
			repaint();
		}
		
	}

}
