/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Normalisation;

import java.util.Arrays;
import java.util.Enumeration;
import java.util.Vector;

import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataCollection;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataSet;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Probe;

public class LowessNormalisation implements Normalisation, Runnable {

	private Vector listeners = new Vector();
	private DataCollection data;
	private final static double log2 = Math.log(2);
	
	public void normalise(DataCollection d) {
		data = d;
		Thread t = new Thread(this);
		t.start();
	}

	public void addNormalisationProgressListener(NormalisationProgressListener l) {
		listeners.add(l);
	}

	public boolean isSuitable(DataCollection d) {
		return true;
	}

	public void run() {
		Probe [] probes = data.probeSet().getAllProbes();
		DataSet [] sets = data.getAllDataSets();

		for (int s=0;s<sets.length;s++) {
			Enumeration e = listeners.elements();
			while (e.hasMoreElements()) {
				((NormalisationProgressListener)e.nextElement()).updateNormalisationProgress("Doing Lowess Normalisation for "+sets[s].name());
				try {
					Thread.sleep(10);
				}
				catch(InterruptedException ie){}
			}
			normaliseSet(sets[s],probes);
		}

		Enumeration e = listeners.elements();
		while (e.hasMoreElements()) {
			((NormalisationProgressListener)e.nextElement()).normalisationComplete();
		}
	}
	
	private void normaliseSet (DataSet s, Probe [] probes) {
		// The first thing we need to do is to get an ordered set of measures
		// for the values for this probeset.  We do the Lowess on an MA plot
		// so it's the M and A values we need.
		
		ProbeMeasure [] measures = new ProbeMeasure[probes.length];
		for (int i=0;i<probes.length;i++) {
			double m = Math.log(s.getSignalValueForProbe(probes[i])/s.getControlValueForProbe(probes[i]))/log2;
			double a = (Math.log(s.getSignalValueForProbe(probes[i])*s.getControlValueForProbe(probes[i]))/log2)/2d;
//			System.out.println("i="+i+" m="+m+" a="+a);
			measures[i] = new ProbeMeasure(probes[i],m,a);
		}
		
		Arrays.sort(measures);
		
		// Now we need to take a sample of all of the probe measures and work
		// out the Lowess regression at that point.

		// We're working on 100 sample points in a distribution.
		int sampleInterval = measures.length/100;
		if (sampleInterval < 1) {
			sampleInterval = 1;
		}
		
		Vector lowessPoints = new Vector();
		
		for (int m=0;m<measures.length;m+=sampleInterval) {
			// We need to get the subset of measures we're going to use
			// for this normalisation.  We're using 40% of the data to
			// calculate the smoothing hence all the divide by 5s.
			
			// A nice summary of how to do the lowess curve is at www.itl.nist.gov/div898/handbook/pmd/section1/dep/dep144.htm
			
			
			int startIndex = (int)(m - ((double)measures.length/5));
			int endIndex = (int)(m + ((double)measures.length/5));

			// Correct if we fall off the end
			if (startIndex < 0) {
				startIndex = 0;
				endIndex = (int)(measures.length/2.5);
			}
			if (endIndex >= measures.length) {
				endIndex = measures.length -1;
				startIndex = endIndex - (int)(measures.length/2.5);
			}
			
//			System.out.println("For point "+m+" using region "+startIndex+" (a="+measures[startIndex].a+") -"+endIndex+" (a="+measures[endIndex].a+") from "+measures.length);

			// Calculate the correlation coefficient
			
			double maxDist = 0;
			for (int w=startIndex;w<=endIndex;w++) {
				measures[w].distance = measures[w].a-measures[m].a;
				if (measures[w].distance < 0) measures[w].distance = 0-measures[w].distance;
				if (measures[w].distance > maxDist) maxDist = measures[w].distance;
			}
//			System.out.println("Calcualted distances");
			
			
			// Scale the distances by the maximum distance in the set and calucalate the weight
			// and do the weighted regression to get the slope and intercept
			double mWeightSum=0;
			double aWeightSum=0;
			double weightSum=0;
			for (int w=startIndex;w<=endIndex;w++) {
				double sd = measures[w].distance/maxDist;
				measures[w].weight= Math.pow((1d - Math.pow(sd,3)),3);
				
				mWeightSum += measures[w].m * measures[w].weight;
				aWeightSum += measures[w].a * measures[w].weight;
				weightSum += measures[w].weight;
			}
			
			double mWeightMean = mWeightSum/weightSum;
			double aWeightMean = aWeightSum/weightSum;
						
			// We can now calculate the slope of the regression.  This is a standard
			// weighted linear regression.
			double slopeNumerator = 0;
			double slopeDenominator = 0;
			
			for (int w=startIndex;w<=endIndex;w++) {
				slopeNumerator += measures[w].weight * ((measures[w].m - mWeightMean)*(measures[w].a-aWeightMean));
				slopeDenominator += measures[w].weight * Math.pow((measures[w].a - aWeightMean),2);
			}
			
			double slope = slopeNumerator/slopeDenominator;
			
			// A big problem we found with Lowess is that if you have part of your dataset
			// with no correlation between M and A then you get a flat regression line.  This
			// allows huge changes in the predicted value and gives wierd normalisation results.
			// We therefore say that the absolute value of the slope must be >0.2 (square >0.04)
			// to use each point.  It doesn't matter if we lose a few points as we interpolate
			// between them anyway.
//			if (Math.pow(slope,2)<0.04) {
//				continue;
//			}
			
			// And then the intercept
			double intercept = mWeightMean - (slope*aWeightMean);
			
//			System.out.println("Slope="+slope+" intecept="+intercept);
			
			// Finally we can get the corrected control value for this point and assign it
			
			double [] point = new double[2];
			point[0]=(slope*measures[m].a)+intercept;
			point[1]=measures[m].a;
//			System.out.println(""+m+"\t"+point[0]+"\t"+point[1]+"\t"+slope);
			lowessPoints.add(point);
			
		}

		// Now we need to go through all of the data correcting the experiment
		// values using the closest Lowess point available.  Basically in the
		// Lowess part we've calculated a set of M offsets which we need to 
		// subtract from our real M values.  We use the A values to determine
		// which is the closest lowess point, and then use that M offset.
		
		// Since we're moving through A in a stepwise mannar we don't have
		// to search all lowess points, just remember which are the nearest.
		Enumeration e =lowessPoints.elements();
		double [] previousPoint = (double [])e.nextElement();
		double [] nextPoint = (double [])e.nextElement();
		
		for (int m=0;m<measures.length;m++) {
			if ((measures[m].a > nextPoint[1]) && e.hasMoreElements() ) {
				previousPoint = nextPoint;
				nextPoint = (double [])e.nextElement();
			}
			
			double mDiff = previousPoint[0] + ((nextPoint[0]-previousPoint[0])*((measures[m].a-previousPoint[1])/(nextPoint[1]-previousPoint[1])));

			if (measures[m].a > nextPoint[1]) {
				// We're working past the last lowess point
				mDiff = nextPoint[0];
			}
			
	
//			System.out.println("M="+m+" Probe="+measures[m].p+" this a="+measures[m].a+" current a="+currentPoint[1]+" next a="+nextPoint[1]+" diff="+currentDiff+" next Diff="+nextDiff);
			
			// Now we know the closest point we can do the correction.
			double newM = measures[m].m-mDiff;
//			System.out.println("Old M:"+measures[m].m+" corrected to "+newM);
			s.setSignalForProbe(measures[m].p,Math.pow(2,newM)*s.getControlValueForProbe(measures[m].p));

		}
	}
	
	public String toString () {
		return shortDescription();
	}

	public String shortDescription() {
		return "Per Array Lowess Normalisation";
	}

	public String longDescription() {
		return "<html>Performs an intensity dependent normalisation<br>of the data to correct dye bias</html>";
	}


	private class ProbeMeasure implements Comparable {

		public Probe p;
		private double m;
		private double a;
		public double weight;
		public double distance;
		
		public ProbeMeasure (Probe p, double m, double a) {
			this.p = p;
			this.m = m;
			this.a = a;
		}

		public int compareTo(Object o) {
			return Double.compare(a,((ProbeMeasure)o).a);
		}
		
	}
	
}
